%% Calculates the load sharing in the process plant.

function [theta0,V,p,q] = bus(Pbus,Qbus,Genset3Connected, x,V0old,Sb,r,xs,e)
%% Extract values:
theta1=x(1); omega1=x(2); e1=e(1);
theta2=x(5); omega2=x(6); e2=e(2);
theta3=x(9); omega3=x(10); e3=e(3);

%% Find base power
if(Genset3Connected)
    Sb_bus = sum(Sb);
else
    Sb_bus = sum(Sb(1:2));
end

%% Find the impedance of the load
ZL = V0old^2/(Pbus-Qbus*1j)*Sb_bus;

%% Calculate the complex value of the induced voltage
e1 = e1*exp(1j*theta1);
e2 = e2*exp(1j*theta2);
e3 = e3*exp(1j*theta3);

%% Calculate the impedance of each generator
z1 = r(1)+1j*omega1*xs(1);
z2 = r(2)+1j*omega2*xs(2);
z3 = r(3)+1j*omega3*xs(3);

%% Calculate the Thevenin voltage
if(Genset3Connected)
    A = [0 Sb(1)  Sb(2)     Sb(3)
    1 sqrt(3)*z1 0  0
    1 0 sqrt(3)*z2  0
    1 0 0   sqrt(3)*z3];
    b = [0;e1;e2;e3];
else
    A = [0 Sb(1)  Sb(2)
    1 sqrt(3)*z1 0
    1 0 sqrt(3)*z2];
    b = [0;e1;e2];
end

x = A\b;
VT = x(1);

%% Calculate the Thevenin imepdance
if(Genset3Connected)
    ZT = 1/(Sb(1)/z1+Sb(2)/z2+Sb(3)/z3)*Sb_bus;
else
    ZT = 1/(Sb(1)/z1+Sb(2)/z2)*Sb_bus;
end

%% Calculate the bus voltage
V0 = ZL/(ZL+ZT)*VT;

%% Calculate the complex power sharing
S1 = V0*((e1-V0)/z1)';
S2 = V0*((e2-V0)/z2)';
S3 = V0*((e3-V0)/z3)';
pe1 = real(S1);
pe2 = real(S2);
pe3 = real(S3);
qe1 = imag(S1);
qe2 = imag(S2);
qe3 = imag(S3);

if(~Genset3Connected)
    pe3 = 0;
    qe3 = 0;
end

p = [pe1; pe2; pe3];
q = [qe1; qe2; qe3];

%% Calculate the electrical angle
theta0 = angle(V0);

%% Calculate the rms bus Voltage
V = abs(V0);
if (V <=0.0001)
    V = 0.0001;
end