%% Calculate constraints
function [G g_L g_U] = findConstraints(psi,phi,umin,umax,wmin,wmax,n,m,Nz,z0,udmax,udmaxnow)
%% Calculate constraints on the form: g1_L < G [z(i); z(i-1)] < g1_L
% First n rows are hard min/max fuel rack position constraints
% n+1:n rows are hard max rate for fuel rack position constraints
% 2nd and 3rd last rows are soft frequency constraints
% Last row is slack variable constraint (\eps >=0)
% z[k+i-1] eps[k+i-1] z[k+i] eps[k+i]
G1 = [zeros(n,3+n) eye(n,n) zeros(n,1)
    zeros(n,1) -eye(n,n) zeros(n,2) eye(n,n) zeros(n,1)
    zeros(1,n+2) 1 zeros(1,n) 1
    zeros(1,n+2) 1 zeros(1,n) -1
    zeros(1,2*n+3) 1];

g1_U = [umax; udmax;
    inf
    wmax;
    inf];

g1_L = [umin;-inf *ones(n,1);
    wmin;
    -inf;
    0];


Nconstraints = n*2+3; % Number of constraints per time instant
Nstates = Nz; % Number of states per time instant

%% Combine the constraints for all time steps to g_L < G Z < g_U
G = zeros(Nconstraints*(m),Nstates*(m+1));
g_L = zeros(Nconstraints*(m),1);
g_U = g_L;
% Apply constaints to all time instance
for i = 0:(m-1)
    G((i*Nconstraints)+(1:Nconstraints),i*Nstates+(1:2*Nstates)) = G1;
    g_L(i*Nconstraints+(1:Nconstraints)) = g1_L;
    g_U(i*Nconstraints+(1:Nconstraints)) = g1_U;
end

%% Add available step increase of fuel rate,
% for the air dependent fuel rate constraint
g_U(n+(1:n))= udmax + udmaxnow;

end

