%% Calculates the optimal no-load-frequency

function [dw0opt PHI PSI dw] = findInput(X,Ad,Bd,B0d,Cu,Du,Adw,Bdw,B0dw,Cuw,Duw,AdFLRp,pFLR,Rn,Qn,Rw,Qw,ud,udw,u0,umaxnow,umin,umax,udmax,wmin,wmax,wminw,wmaxw,p,Sb,genset3connected,time,debuggingfile,useUMaxNow)
%% Parameters
m = 30; % Length of control horizon
n = 3; % number of gensets
Nopt = n+1; % Number of free variables (variables to be optimized) (No load frequency, slack variable)
Nz = n+2; % number of z states (frequency, fuel rack positions, slack variable)
Nx = 2+4*n; % Number of help states

%% Find psi, phi and T for the worst case:
Cw = [0 0 0 1/2 0 0 0 1/2 0 0
    Cuw
    zeros(1,2+4*(n-1))];
Dw = [zeros(1,n-1) 0
    Duw zeros(n-1,1)
    zeros(1,n-1) 0];
zdw = [1
    udw
    0];
z0w = [mean(X(4:4:4*(n-1)))
    u0(1:end-1)
    0];
udmaxnow = useUMaxNow*(umaxnow-u0);

if(genset3connected)
    [psiw phiw Tw] = findPsiTWorstCase(m,n-1,Nz-1,Nopt-1,Nx-4,Bd,B0d,Adw,Bdw,B0dw,Cw,Dw,AdFLRp,pFLR,z0w,zdw);
else
    [psiw phiw Tw] = findPsiT(m,n-1,Nz-1,Nopt-1,Nx-4,Adw,Bdw,B0dw,Cw,Dw,z0w,zdw);
end
%% Expand cost of changing input, for slack variable
Rw = [Rw zeros(n-1,1)
    zeros(1,n-1) 0];

if (genset3connected)
    %% %%%% Generator 3 is connected %%%%%%%%%%%%%
    %% Expands cost functions to include all time steps
    Rw = blockdiag2(Rw,m-1);
    Qw = blockdiag2(Qw,m-1);
    %% Find constraints for failure case
    [Gw g_Lw g_Uw] = findConstraints(psiw,phiw,umin(1:(end-1))...
        ,umax(1:(end-1)),wminw,wmaxw,n-1,m-1,Nz-1,z0w,udmax(1:(end-1)),udmaxnow(1:(end-1)));
    %% Extra constrints for first step
    [Gwe g_Lwe g_Uwe] = findExtraConstraints(psiw,phiw,umin(1:(end-1))...
        ,umax(1:(end-1)),wminw,wmaxw,n-1,m-1,Nz-1,z0w,udmax(1:(end-1)),udmaxnow(1:(end-1)));
    
    %% Normal case
    C = [0 0 0 1/3 0 0 0 1/3 0 0 0 1/3 0 0
        Cu
        zeros(1,2+4*n)];
    D = [zeros(1,n) 0
        Du zeros(n,1)
        zeros(1,n) 0];
    %% Desired z
    zd = [1
        ud
        0];
    %% Initial z
    z0 = [mean(X(4:4:4*n))
        u0
        0];
    
    %% Find psi, phi and T for the nominal case
    [psi phi T] = findPsiT(m,n,Nz,Nopt,Nx,Ad,Bd,B0d,C,D,z0,zd);
    
    %% Merge normal and worst case:
    [xPsiN yPsiN] = size(psi);
    [xPsiW yPsiW] = size(psiw);
    PSI = [psi zeros(xPsiN,yPsiW-Nopt+1)
        psiw(:,1:(Nopt-1))  zeros(xPsiW,1+ yPsiN -Nopt)  psiw(:,Nopt:end)];
    PHI = [phi;phiw];
    Tc = [T;Tw];
    
    %% Expand the cost function for nominal case:
    QN = blockdiag2(Qn,m+1);
    Rn = [Rn zeros(n,1)
        zeros(1,n) 0];
    RN = blockdiag2(Rn,m);
    
    %% Merge cost functions
    [xN yN] = size(QN);
    [xW yW] = size(Qw);
    Q = [QN zeros(xN,yW)
        zeros(xW,yN) Qw];
    [xN yN] = size(RN);
    [xW yW] = size(Rw);
    R = [RN zeros(xN,yW)
        zeros(xW,yN) Rw];
    
    
    %% Get the constraints
    [G g_L g_U] = findConstraints(psi,phi,umin,umax,wmin,wmax,n,m,Nz,z0,udmax,udmaxnow);
    
    %% Merge nominal and worst case:
    [xN yN] = size(G);
    [xW yW] = size(Gw);
    G = [G zeros(xN,yW-Nz+1)
        zeros(xW,Nz) Gw(:,1:(Nz-2)) zeros(xW, 1) Gw(:,Nz-1) zeros(xW, yN -2*Nz)  Gw(:,Nz:end)
        Gwe(:,1:(Nz-2)) zeros(2, 1) Gwe(:,Nz-1)  zeros(2, yN-Nz) Gwe(:,Nz:end) zeros(2, yW-(Nz-1)*2) ];
    
    g_L = [g_L; g_Lw; g_Lwe];
    g_U = [g_U; g_Uw; g_Uwe];
    dw = zeros((n+1)*(m)+n*(m-1),1);
else
    %% %%%%%% Genset 3 is disconnected %%%%%%%%%%%%%%%%
    Rw = blockdiag2(Rw,m);
    Qw = blockdiag2(Qw,m+1);
    % Find constraints for worst case
    [Gw g_Lw g_Uw] = findConstraints(psiw,phiw,umin(1:(end-1))...
        ,umax(1:(end-1)),wminw,wmaxw,n-1,m,Nz-1,z0w,udmax(1:(end-1)),udmaxnow(1:(end-1)));
    Q = Qw;
    R = Rw;
    PSI = psiw;
    PHI = phiw;
    G = Gw;
    g_L = g_Lw;
    g_U = g_Uw;
    Tc = Tw;
    dw = zeros(n*(m+1),1);
end
%% Solve the qp problem:
coder.extrinsic('qpAssign','tomRun','extractResult');

%% convert constraints from z variables to optimalization variables
Omega = G*PSI;
omega_L = g_L-G*PHI;
omega_U = g_U-G*PHI;

Prob = qpAssign(2*(R+PSI'*Q*PSI), -2*PSI'*Q*Tc, Omega, omega_L, omega_U, [], [], [], [],...
    [], [], [], [], []);

Result = tomRun('SQOPT', Prob, 0);

%% Extract the result from tomrun.
dw = extractResult(Result);

%% Make debuggings file
debuggConstraints(Omega,omega_L,omega_U,dw,m,PSI,PHI,time,genset3connected,debuggingfile);

%% Extracts values used as new configuration of governors
if (genset3connected)
    dw0opt = dw(1:n);
    return;
else
    dw0opt = [dw(1:(n-1)); 0];
    return;
end
end