version = 4;

%% Genset parameters
N = 2;
w_b = 60*2*pi;
w_bm = w_b*2/N;
wb = w_b;
tau_e = 0.9/w_bm;
Sb = [5e6;5e6;10e6];
Sb_bus = sum(Sb);
Tb = Sb_bus/w_b;
V_b = 11e3;
r = [.003;.0025;.002];
xs = [1;1.5;2];
H = [10; 7; 6];
D = [.02;.025;.03];

%% Governor parameters
ku = 1+D+r;
Kp = [10 10 10]';
Ki = [5 5 5]';
Kd = [5 5 5]';
N = 10*[1 1 1]';
Droop = [.05; .05;.05];
wNLstart = 1.025*[1;1;1];
w0 = 1+Droop/2+[0.02; 0; -0.01];
uMax = 1.25*[1;1;1];
uMin = 0*[1;1;1];
uRampTime = [1; 1; 1];

%% Turbo parameters
AFlow = 14*[1; 1; 1];
AFhigh = 23*[1; 1; 1];
AFn = 27*[1; 1; 1];
AFlowc = 23*[1; 1; 1];
ma0 = .2*[1; 1; 1];
Tturbo = 5*[1;1;1];

%% AVR parameters
KvP = 400*[1;1;1];
KvI = 100*[1;1;1];
DroopAvr = [.05;.05;.05];
v0 = [1;1;1];
vfMax = 20*[1;1;1];
vfMin = 0.01*[1;1;1];
Avr0 = 1*[1;1;1];
Tvf = .001*[1;1;1];

%% Low pass for bus voltage
TV0 = 0.001; % Time constant for low pass filter for calculation of bus voltage
VT0 = 1; % Initial voltage on bus



%% Set initial conditions
theta1 = 0;
theta2 = 0;
theta3 = 0;

omega1 = .99;
omega2 = .99;
omega3 = .99;
omega_turbo_0 = .9*[1;1;1];

samplingTimeSolver = 0.0001;
samplingsTime = 1;

%% Fast load reduction
tFLR = 0.2; % Seconds before load reduction appear.
 % Power which is reduced by the FLR as a ratio of the power from disconnected genset 
 %(0 is no reduction, 1 reduces the power at the bus with the same amount as genset 3 was delivering)

%% MPC
% First update of MPC
t0 = .01;  
% Desired load sharing, must satisfy: ud'*Sb=Sb_bus
ud = [1.1;.9;1];

%% Cost functions for normal case
% Cost for control output
%        omega, u1, u2, u3, slack variable for frequency error
Qn = diag([1 .01 .01 .02 100]);

% Cost of changing no load frequency
Rn = 1e-1*diag([1 1 1]);

%% Cost function for worst case
%             omega, u1, u2, slack variable for frequency error
Qw = diag([0 0 0 100]);

% Cost of changing no load frequency
Rw = 1e-1*diag([1 1]);

%% Constraints for MPC
uMaxMPC = 1.1*[1;1;1];
uMinMPC = .01*[1;1;1];
udMaxMPC = 1/30*[1;1;1]*samplingsTime;
wmin = .95;
wmax = 1.05;
wminw = 0.96;
wmaxw = 1.05;