%% Establish the state matrices for the system with three genset
function [A B B0 Cu Du EinvF ud] = makeLinearSystem(H,D,p,Sb_bus,Sb,q,x,vf,vt,theta0,ku,xs,u,wb,r,KvP,KvI,Kp,Ki,Kd,Droop,DroopAvr,w0,v0,samplingsTime,ud)
n = 3-1; % number of gensets -1
m = 4;   % number of states for each genset
delta = x(3:m:(3+m*n))-theta0*[1;1;1];
w = x(4:m:(4+m*n));
xi = x(5:m:(5+m*n));
h = 1./(2*H+ku.*Kd);
alpha_w = h.*(-D+1./w.^2.*(p+(p.^2+q.^2).*r/vt)-ku.*Kp);
alpha_p = -h./w.*(1+2.*p.*r./vt^2)-h.*ku.*Kp.*w0.*Droop;
alpha_q = -h./w.*(2*q.*r./vt^2);
alpha_v = 2.*h.*(p.^2+q.^2).*r./(w.*vt.^3);

gamma_Z = vf.*vt.*cos(delta)./xs;
beta_Z = -vt.*vf.*sin(delta)./xs;
gamma_vt = sin(delta)./xs.*(KvP.*vt-vf);
beta_vt = -vf./xs.*cos(delta)+2.*vt./xs./w+vt.*cos(delta)./xs.*KvP;
gamma_q = KvP.*DroopAvr.*vt./xs.*sin(delta);
beta_q = 1+vt./xs.*KvP.*DroopAvr.*cos(delta);
gamma_phi = vf.*vt./xs.*cos(delta);
beta_phi = -vt.*vf.*sin(delta)./xs;
beta_w = vt.^2./(xs.*w.^2);
gamma_xiv = KvI.*vt./xs.*sin(delta);
beta_xiv = KvI.*vt./xs.*cos(delta);

kp = h.*ku.*Kp.*(1-p.*Droop);
ki = h.*ku.*Ki;
wref = w0.*(1-Droop.*p);
verr = (1-DroopAvr.*q)-vt;
wd0 = h.*((-D-ku.*Kp).*w+ku.*Kp.*wref+ku.*Ki.*xi-1./w.*(p+(p.^2+q.^2).*r./vt^2));

E = [0 0 Sb(1)/Sb_bus 0 Sb(2)/Sb_bus 0 Sb(3)/Sb_bus 0
    0 0 0 Sb(1)/Sb_bus 0 Sb(2)/Sb_bus 0 Sb(3)/Sb_bus
    gamma_Z(1) gamma_vt(1) 1 gamma_q(1) 0 0 0 0
    beta_Z(1) beta_vt(1) 0 beta_q(1) 0 0 0 0
    gamma_Z(2) gamma_vt(2) 0 0 1 gamma_q(2) 0 0
    beta_Z(2) beta_vt(2) 0 0 0 beta_q(2) 0 0
    gamma_Z(3) gamma_vt(3) 0 0 0 0 1 gamma_q(3)
    beta_Z(3) beta_vt(3) 0 0 0 0 0 beta_q(3)];

F = [1 0 zeros(1,12)
    0 1 zeros(1,12)
    0 0 gamma_phi(1) 0 0 gamma_xiv(1) zeros(1,8)
    0 0 beta_phi(1) beta_w(1) 0 beta_xiv(1) zeros(1,8)
    0 0 zeros(1,4) gamma_phi(2) 0 0 gamma_xiv(2) zeros(1,4)
    0 0 zeros(1,4) beta_phi(2) beta_w(2) 0 beta_xiv(2) zeros(1,4)
    0 0 zeros(1,8) gamma_phi(3) 0 0 gamma_xiv(3)
    0 0 zeros(1,8) beta_phi(3) beta_w(3) 0 beta_xiv(3)];

B0 = [0
    0
    wb*(2/3*w(1)-1/3*w(2)-1/3*w(3))
    wd0(1)
    wref(1)-w(1)
    verr(1)
    wb*(2/3*w(2)-1/3*w(1)-1/3*w(3))
    wd0(2)
    wref(2)-w(2)
    verr(2)
    wb*(2/3*w(3)-1/3*w(2)-1/3*w(1))
    wd0(3)
    wref(3)-w(3)
    verr(3)];
Atilde = [  zeros(2,14)
    0 0 0 wb*2/3 0 0 0 -wb/3 0 0 0 -wb/3 0 0
    0 0 0 alpha_w(1) ki(1) 0 zeros(1,8)
    0 0 0 -1 0 0  zeros(1,8)
    zeros(1,14)
    0 0 0 -wb/3 0 0 0 wb*2/3 0 0 0 -wb/3 0 0
    0 0 zeros(1,4) 0 alpha_w(2) ki(2) 0 zeros(1,4)
    0 0 zeros(1,4) 0 -1 0 0  zeros(1,4)
    zeros(1,14)
    0 0 0 -wb/3 0 0 0 -wb/3 0 0 0 wb*2/3 0 0
    0 0 zeros(1,8) 0 alpha_w(3) ki(3) 0
    0 0 zeros(1,8) 0 -1 0 0
    zeros(1,14)];

B = [zeros(3,3)
    kp(1) 0 0
    (1-Droop(1)*p(1)) 0 0
    zeros(2,3)
    0 kp(2) 0
    0 (1-Droop(2)*p(2)) 0
    zeros(2,3)
    0 0 kp(3)
    0 0 (1-Droop(3)*p(3))
    zeros(1,3)];

K = [zeros(3,8)
    0 alpha_v(1) alpha_p(1) alpha_q(1) 0 0 0 0
    0 0 -w0(1)*Droop(1) 0 0 0 0 0
    0 -1 0 -DroopAvr(1) 0 0 0 0
    0 0 0 0 0 0 0 0
    0 alpha_v(2) 0 0 alpha_p(2) alpha_q(2) 0 0
    0 0 0 0 -w0(2)*Droop(2) 0 0 0
    0 -1 0 0 0 -DroopAvr(2) 0 0
    0 0 0 0 0 0 0 0
    0 alpha_v(3) 0 0 0 0 alpha_p(3) alpha_q(3)
    0 0 0 0 0 0 -w0(3)*Droop(3) 0
    0 -1 0 0 0 0 0 -DroopAvr(3)];

Einv = inv(E);
%% Nominal system
A = K*Einv*F + Atilde;
u = Kp.*(wref-w)+Ki.*xi+Kd.*wd0;
Kp = diag(Kp);
Ki = diag(Ki);
Kd = diag(Kd);

H_omega = [0 0 0 1 0 0 zeros(1,8)
    0 0 zeros(1,4) 0 1 0 0 zeros(1,4)
    0 0  zeros(1,8) 0 1 0 0];
H_dp = [0 0 1 0 zeros(1,4)
    0 0 0 0 1 0 0 0
    0 0 zeros(1,4) 1 0];
Cup = -Kp*diag(w0.*Droop)*H_dp*Einv*F-Kp*H_omega;
Dup = Kp*diag(ones(n+1,1)-p.*Droop);
Cui = Ki*[0 0 0 0 1 0 zeros(1,8)
    0 0 zeros(1,4) 0 0 1 0 zeros(1,4)
    0 0 zeros(1,8) 0 0 1 0];
Cud = -Kd*H_omega*A;
Dud = -Kd*H_omega*B;
Cu = Cup+Cui+Cud;
Du = Dup+Dud;
EinvF = Einv*F;

%% Set desired ud
ud = ud*p'*Sb/sum(Sb);
end

