%% Establish state space equation for system with two generators.
function [A B B0 Cu Du EinvFW ud] = makeLinearWorstCaseSystem(H,D,p,Sb_bus,Sb,q,x,vf,vt,theta0,ku,xs,u,wb,r,KvP,KvI,Kp,Ki,Kd,Droop,DroopAvr,w0,v0,samplingsTime,genset3connected,pFLR,ud)
n = 3-1; % number of gensets -1
m = 4;   % number of states for each genset
w = x(4:m:(4+m*n));
xi = x(5:m:(5+m*n));
delta = x(3:m:(3+m*n))-theta0*[1;1;1];

if(genset3connected)
    %% Calculate load sharing after disconnection
    % Assume that both generator take equal share of excess load.
    pexcess = p(3)*Sb(3)-pFLR*Sb_bus;
    p = [p(1)+pexcess/Sb(1)/2
        p(2)+pexcess/Sb(2)/2
        0];
    q = [q(1)+q(3)*Sb(3)/Sb(1)/2
        q(2)+q(3)*Sb(3)/Sb(2)/2
        0];
    % Complex power:
    s = p+1j*q;
    v = 1;
    theta0 = 0;
    z = r+1j*xs;
    e = conj(s./v).*z+v;
    vf = abs(e)./w;
    delta = angle(e); 
end

% Calculate desired u
ud = ud*(p'*Sb)/sum(Sb(1:2));

h = 1./(2*H+ku.*Kd);
alpha_w = h.*(-D+1./w.^2.*(p+(p.^2+q.^2).*r/vt)-ku.*Kp);
alpha_p = -h./w.*(1+2.*p.*r./vt^2)-h.*ku.*Kp.*w0.*Droop;
alpha_q = -h./w.*(2*q.*r./vt^2);
alpha_v = 2.*h.*(p.^2+q.^2).*r./(w.*vt.^3);


gamma_Z = vf.*vt.*cos(delta)./xs;
beta_Z = -vt.*vf.*sin(delta)./xs;
gamma_vt = sin(delta)./xs.*(KvP.*vt-vf);
beta_vt = -vf./xs.*cos(delta)+2.*vt./xs./w+vt.*cos(delta)./xs.*KvP;
gamma_q = KvP.*DroopAvr.*vt./xs.*sin(delta);
beta_q = 1+vt./xs.*KvP.*DroopAvr.*cos(delta);
gamma_phi = vf.*vt./xs.*cos(delta);
beta_phi = -vt.*vf.*sin(delta)./xs;
beta_w = vt.^2./(xs.*w.^2);
gamma_xiv = KvI.*vt./xs.*sin(delta);
beta_xiv = KvI.*vt./xs.*cos(delta);

kp = h.*ku.*Kp.*(1-p.*Droop);
ki = h.*ku.*Ki;
wref = w0.*(1-Droop.*p);
verr = (1-DroopAvr.*q)-vt;
wd0 = h.*((-D-ku.*Kp).*w+ku.*Kp.*wref+ku.*Ki.*xi-1./w.*(p+(p.^2+q.^2).*r./vt^2));

E = [0 0 Sb(1)/Sb_bus 0 Sb(2)/Sb_bus 0
    0 0 0 Sb(1)/Sb_bus 0 Sb(2)/Sb_bus
    gamma_Z(1) gamma_vt(1) 1 gamma_q(1) 0 0
    beta_Z(1) beta_vt(1) 0 beta_q(1) 0 0
    gamma_Z(2) gamma_vt(2) 0 0 1 gamma_q(2)
    beta_Z(2) beta_vt(2) 0 0 0 beta_q(2)];

F = [1 0 zeros(1,8)
    0 1 zeros(1,8)
    0 0 gamma_phi(1) 0 0 gamma_xiv(1) zeros(1,4)
    0 0 beta_phi(1) beta_w(1) 0 beta_xiv(1) zeros(1,4)
    0 0 zeros(1,4) gamma_phi(2) 0 0 gamma_xiv(2)
    0 0 zeros(1,4) beta_phi(2) beta_w(2) 0 beta_xiv(2)];



B0 = [0
    0
    wb*(2/3*w(1)-1/3*w(2)-1/3*w(3))
    wd0(1)
    wref(1)-w(1)
    verr(1)
    wb*(2/3*w(2)-1/3*w(1)-1/3*w(3))
    wd0(2)
    wref(2)-w(2)
    verr(2)];
Atilde = [  zeros(2,10)
    0 0 0 wb/2 0 0 0 -wb/2 0 0
    0 0 0 alpha_w(1) ki(1) 0 zeros(1,4)
    0 0 0 -1 0 0  zeros(1,4)
    zeros(1,10)
    0 0 0 -wb/2 0 0 0 wb/2 0 0 
    0 0 zeros(1,4) 0 alpha_w(2) ki(2) 0
    0 0 zeros(1,4) 0 -1 0 0
    zeros(1,10)];

B = [zeros(3,2)
    kp(1) 0
    (1-Droop(1)*p(1)) 0
    zeros(2,2)
    0 kp(2)
    0 (1-Droop(2)*p(2))
    zeros(1,2)];

K = [zeros(3,6)
    0 alpha_v(1) alpha_p(1) alpha_q(1) 0 0
    0 0 -w0(1)*Droop(1) 0 0 0
    0 -1 0 -DroopAvr(1) 0 0
    0 0 0 0 0 0
    0 alpha_v(2) 0 0 alpha_p(2) alpha_q(2)
    0 0 0 0 -w0(2)*Droop(2) 0
    0 -1 0 0 0 -DroopAvr(2)];

Einv = inv(E);
%% Nominal system
u = Kp.*(wref-w)+Ki.*xi+Kd.*wd0;
A = K*Einv*F + Atilde;
Kp = diag(Kp(1:2));
Ki = diag(Ki(1:2));
Kd = diag(Kd(1:2));
Droop = Droop(1:2);
w0 = w0(1:2);

H_omega = [0 0 0 1 0 0 zeros(1,4)
    0 0 zeros(1,4) 0 1 0 0];
H_dp = [0 0 1 0 zeros(1,2)
    0 0 0 0 1 0];
Cup = -Kp*diag(w0.*Droop)*H_dp*Einv*F-Kp*H_omega;
Dup = Kp*diag(ones(n,1)-p(1:2).*Droop);
Cui = Ki*[0 0 0 0 1 0 zeros(1,4)
    0 0 zeros(1,4) 0 0 1 0];
Cud = -Kd*H_omega*A;
Dud = -Kd*H_omega*B;
Cu = Cup+Cui+Cud;
Du = Dup+Dud;
EinvFW = Einv*F;

end

